﻿//using System.Data;
//using System.Text;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
    
namespace RenderBest
{
    public partial class Form1 : Form
    {
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
    
            using (Form1 frm = new Form1())
            {
                // Показать форму перед созданием движка
                frm.Show();
                // Создать и инициализировать графический движок
                // нашей функцией
                frm.InitializeGraphics();
                // Войти в цикл сообщений
                Application.Run(frm);
            }
        }
    
        public Form1()
        {
            InitializeComponent();
    
            // Устраняем мерцание окна при бесконечной перерисовке
            this.SetStyle(ControlStyles.AllPaintingInWmPaint |
                ControlStyles.Opaque, true);
    
            // Инициализация генератора случайных чисел
            rand = new System.Random();
        }
    
        private Device device = null;       // Графический конвейер
        private VertexBuffer vb = null;     // Вершинный буфер
        private const int NumberItems = 12; // Количество вершин объекта
        System.Random rand;                 // Поле для генератора случайных чисел
    
        public void InitializeGraphics()
        {
            // Создание объекта и настройка параметров представления
            // Создать объект параметров представления
            PresentParameters presentParams = new PresentParameters();
            // Установить оконный режим
            presentParams.Windowed = true;
            // Сбрасывать содержимое буфера, если он не готов к представлению
            presentParams.SwapEffect = SwapEffect.Discard;
            // Создать объект устройства и сохранить ссылку на него
            device = new Device(0, DeviceType.Hardware, this,
                CreateFlags.SoftwareVertexProcessing, presentParams);
    
            // Создать вершинный буфер
            vb = new VertexBuffer(typeof(CustomVertex.PositionColored),
                                  NumberItems,
                                  device,
                                  Usage.Dynamic | Usage.WriteOnly,
                                  CustomVertex.PositionColored.Format,
                                  Pool.Default);
    
            // Регистрация события Created вершинного буфера
            vb.Created += new EventHandler(vb_Created);
    
            // Принудительный вызов создания треугольника 
            // и заполнения вершинного буфера при первом 
            // создании формы
            vb_Created(vb, null);
        }
    
        void vb_Created(object sender, EventArgs e)
        {
            // Определить внутреннюю ссылку на вершинный буфер
            VertexBuffer buffer = (VertexBuffer)sender; // Явное приведение типов
    
            // Создать локальный массив ссылок для вершин объекта
            CustomVertex.PositionColored[] verts = new
                CustomVertex.PositionColored[NumberItems];
    
            // Сгенерировать случайно координаты и цвета вершин
            double factor = 2.5d;// Весовой коэффициент
            for (int i = 0; i < NumberItems; i++)
            {
                float xPos = (float)((rand.NextDouble() - rand.NextDouble()) * factor);
                float yPos = (float)((rand.NextDouble() - rand.NextDouble()) * factor);
                float zPos = (float)((rand.NextDouble() - rand.NextDouble()) * factor);
                verts[i].SetPosition(new Vector3(xPos, yPos, zPos));
                verts[i].Color = System.Drawing.Color.FromArgb(rand.Next(256),
                                                               rand.Next(256),
                                                               rand.Next(256)).ToArgb();
            }
    
            // Заполнить вершинный буфер данными треугольников
            buffer.SetData(verts, 0, LockFlags.None);
        }
    
        // Флаг регенерации вершин перед началом
        // каждого нового цикла обхода всех примитивов.
        // Первое заполнение вершинного буфера выполнено
        // в InitializeGraphics() при запуске приложения
        private bool needRecreate = false;
        // Захват стартового времени в тактах
        private static readonly int beginTickCount =
            System.Environment.TickCount;
    
        protected override void OnPaint(PaintEventArgs e)
        {
            // Очистить цветом клиентскую область формы
            device.Clear(ClearFlags.Target,
                System.Drawing.Color.White,
                1.0F, 0);
    
            // Вызов нашей функции установки камеры
            SetupCamera();
    
            // Сформировать сцену с учетом параметров 
            // "положение-нормаль-цвет"
            device.BeginScene();
            // Установить формат обработки вершин при отображении
            device.VertexFormat = CustomVertex.PositionColored.Format;
            // Нарисовать треугольник по данным из буфера
            device.SetStreamSource(0, vb, 0);
    
            const int countTime = 5 * 1000; // Держать изображение 5 секунд 
            const int countIndex = 6;       // Количество вариантов показа
            int index = ((System.Environment.TickCount - beginTickCount) / countTime)
                % countIndex;
            switch (index)
            {
                case 0: // PointList
                    device.DrawPrimitives(PrimitiveType.PointList, 0, NumberItems);
                    if (needRecreate)// Выполнить только при очередном index == 0
                    {
                        vb_Created(vb, null);// Сгенерировать новый вершинный буфер
                        needRecreate = false;// Сбросить флаг регенерации вершин
                    }
                    this.Text = "Несоединенные точки вершин";
                    break;
                case 1: // LineList
                    device.DrawPrimitives(PrimitiveType.LineList, 0, NumberItems / 2);
                    needRecreate = true;// Приготовить флаг для регенерации вершин при index == 0
                    this.Text = "Соединие пар вершин линиями";
                    break;
                case 2: // LineStrip
                    device.DrawPrimitives(PrimitiveType.LineStrip, 0, NumberItems - 1);
                    this.Text = "Соединение вершин ломаной";
                    break;
                case 3: // TriangleList
                    device.DrawPrimitives(PrimitiveType.TriangleList, 0, NumberItems / 3);
                    this.Text = "Отдельные треугольники";
                    break;
                case 4: // TriangleStrip
                    device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, NumberItems - 2);
                    this.Text = "Непрерывная ферма треугольников";
                    break;
                case 5: // TriangleFan
                    device.DrawPrimitives(PrimitiveType.TriangleFan, 0, NumberItems - 2);
                    this.Text = "Треугольный веер из точки";
                    break;
            }
    
            device.EndScene();
    
            // Показать буфер кадра 
            device.Present();
    
            // Принудительно перерисовать
            this.Invalidate();
        }
    
        // Установка камеры в сцену
        private void SetupCamera()
        {
            //Создание перспективы 
            device.Transform.Projection = Matrix.PerspectiveFovLH(
                (float)Math.PI / 4, // Угол зрения равен 45 градусов
                                    // Форматное соотношение сторон
                (float)this.ClientSize.Width / (float)this.ClientSize.Height, 
                1.0F,               // Ближний план
                100.0F);            // Дальний план
    
            //Добавление камеры 
            device.Transform.View = Matrix.LookAtLH(
                new Vector3(0, 0, 5.0F),    // Положение камеры
                new Vector3(),              // Положение объекта текущее
                new Vector3(0, 1, 0));      // Направление камеры
    
            // Освещение
            device.RenderState.Lighting = false;
    
            // Увеличение масштаба отображения точек вершин объекта
            device.RenderState.PointSize = 5.0f;
        }
    }
}