﻿//using System.Data;
//using System.Text;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
    
namespace RenderBest
{
    public partial class Form3 : Form
    {
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
    
            using (Form3 frm = new Form3())
            {
                // Показать форму перед созданием движка
                frm.Show();
                // Создать и инициализировать графический движок
                // нашей функцией
                frm.InitializeGraphics();
                // Войти в цикл сообщений
                Application.Run(frm);
            }
        }
    
        public Form3()
        {
            InitializeComponent();
    
            // Устраняем мерцание окна при бесконечной перерисовке
            this.SetStyle(ControlStyles.AllPaintingInWmPaint |
                ControlStyles.Opaque, true);
            this.Click += new EventHandler(Form1_Click);
        }
    
        bool flagRotate = true;
        void Form1_Click(object sender, EventArgs e)
        {
            flagRotate = !flagRotate;
        }
    
        private Device device = null;
        private VertexBuffer vb = null;
        private IndexBuffer ib = null;
    
        public void InitializeGraphics()
        {
            // Создание объекта и настройка параметров представления
            // Создать объект параметров представления
            PresentParameters presentParams = new PresentParameters();
            // Установить оконный режим
            presentParams.Windowed = true;
            // Сбрасывать содержимое буфера, если он не готов к представлению
            presentParams.SwapEffect = SwapEffect.Discard;
    
            // Настройка буфера глубины для параметров устройства
            presentParams.EnableAutoDepthStencil = true;
            presentParams.AutoDepthStencilFormat =
                Microsoft.DirectX.Direct3D.DepthFormat.D16;
    
            // Создать объект устройства и сохранить ссылку на него
            device = new Device(0, DeviceType.Hardware, this,
                CreateFlags.SoftwareVertexProcessing, presentParams);
    
            // Создать вершинный буфер
            vb = new VertexBuffer(typeof(CustomVertex.PositionColored),
                                  8,
                                  device,
                                  Usage.Dynamic | Usage.WriteOnly,
                                  CustomVertex.PositionColored.Format,
                                  Pool.Default);
    
            // Регистрация события Created вершинного буфера
            vb.Created += new EventHandler(vb_Created);
    
            // Принудительный вызов создания треугольника 
            // и заполнения вершинного буфера при первом 
            // создании формы
            vb_Created(vb, null);

            // Создать индексный буфер для координат вершин куба
            ib = new IndexBuffer(typeof(short),
                                 indices.Length,
                                 device,
                                 Usage.WriteOnly,
                                 Pool.Default);

            // Регистрация события Created индексного буфера
            // Код написать вручную (для создания обработчика)!!!
            ib.Created += new EventHandler(ib_Created);

            // Принудительный вызов обработчика при первом запуске
            ib_Created(ib, null);
        }


        // Обработчик события повторного заполнения индексного буфера
        void ib_Created(object sender, EventArgs e)
        {
            IndexBuffer buffer = (IndexBuffer)sender;
            buffer.SetData(indices, 0, LockFlags.None);
        }
    
        // Определение индексного массива вершин треугольных примитивов
        private static readonly short[] indices = {
            2,0,1,// Задняя грань:    треугольник 2-0-1 (невидимая сторона - обход левый)
            2,1,3,// Задняя грань:    треугольник 2-1-3 (невидимая сторона - обход левый)
            5,6,4,// Передняя грань:  треугольник 5-6-4 (лицевая сторона - обход правый)
            5,7,6,// Передняя грань:  треугольник 5-7-6 (лицевая сторона - обход правый)
            0,5,4,// Верхняя грань:   треугольник 0-5-4 (лицевая сторона - обход правый)
            0,2,5,// Верхняя грань:   треугольник 0-2-5 (лицевая сторона - обход правый)
            1,6,7,// Нижняя грань:    треугольник 1-6-7 (невидимая сторона - обход левый)
            1,7,3,// Нижняя грань:    треугольник 1-7-3 (невидимая сторона - обход левый)
            0,6,1,// Левая грань:     треугольник 0-6-1 (невидимая сторона - обход левый)
            0,4,6,// Левая грань:     треугольник 0-4-6 (невидимая сторона - обход левый)
            2,3,7,// Правая грань:    треугольник 2-3-7 (лицевая сторона - обход правый)
            2,7,5 // Правая грань:    треугольник 2-7-5 (лицевая сторона - обход правый)
		};
    
        void vb_Created(object sender, EventArgs e)
        {
            // Определить внутреннюю ссылку на вершинный буфер
            VertexBuffer buffer = (VertexBuffer)sender; // Явное приведение типов
    
            // Создать локальный массив структур непреобразованных координат
            CustomVertex.PositionColored[] verts = 
                new CustomVertex.PositionColored[8];
    
            // Задать параметры вершин куба в соответствии с рисунком
            //
            verts[0] = new CustomVertex.PositionColored(-1.0F, 1.0F, 1.0F, Color.Purple.ToArgb());
            verts[1] = new CustomVertex.PositionColored(-1.0F, -1.0F, 1.0F, Color.Red.ToArgb());
            verts[2] = new CustomVertex.PositionColored(1.0F, 1.0F, 1.0F, Color.Blue.ToArgb());
            verts[3] = new CustomVertex.PositionColored(1.0F, -1.0F, 1.0F, Color.Yellow.ToArgb());
            verts[4] = new CustomVertex.PositionColored(-1.0F, 1.0F, -1.0F, Color.Gold.ToArgb());
            verts[5] = new CustomVertex.PositionColored(1.0F, 1.0F, -1.0F, Color.Green.ToArgb());
            verts[6] = new CustomVertex.PositionColored(-1.0F, -1.0F, -1.0F, Color.Black.ToArgb());
            verts[7] = new CustomVertex.PositionColored(1.0F, -1.0F, -1.0F, Color.WhiteSmoke.ToArgb());
    
            // Заполнить вершинный буфер данными треугольников
            buffer.SetData(verts, 0, LockFlags.None);
        }
    
        protected override void OnPaint(PaintEventArgs e)
        {
            // Очистить цветом клиентскую область формы
            device.Clear(ClearFlags.Target 
                | ClearFlags.ZBuffer,
                System.Drawing.Color.CornflowerBlue,
                1.0F, 0);
    
            // Вызов нашей функции установки камеры
            SetupCamera();
    
            // Сформировать сцену с учетом параметров 
            // "положение-нормаль-цвет"
            device.BeginScene();
            // Установить формат обработки вершин при отображении
            device.VertexFormat = CustomVertex.PositionColored.Format;
            // Нарисовать треугольник по данным из буфера
            device.SetStreamSource(0, vb, 0);
            device.Indices = ib;
            DrawCubes();
            device.EndScene();
    
            // Показать буфер кадра 
            device.Present();
    
            // Принудительно перерисовать
            this.Invalidate();
        }

        private float angle = 0;// Закрытый член класса
    
        // Установка камеры в сцену
        private void SetupCamera()
        {
            //Создание перспективы 
            device.Transform.Projection = Matrix.PerspectiveFovLH(
                (float)Math.PI / 4, // Угол зрения равен 45 градусов
                                    // Форматное соотношение сторон
                (float)this.ClientSize.Width / (float)this.ClientSize.Height, 
                1.0F,               // Ближний план
                100.0F);            // Дальний план
    
            //Добавление камеры 
            device.Transform.View = Matrix.LookAtLH(
                new Vector3(0, 0, 15.0F),    // Положение камеры
                new Vector3(),              // Положение объекта текущее
                new Vector3(0, 1, 0));      // Направление камеры
    
            // Освещение
            device.RenderState.Lighting = false;
        }
    
        private void DrawCubes()
        {
            //
            // Средний ряд
            //
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 2.0f,
                angle / (float)Math.PI / 4.0f) *
                Matrix.Translation(0.0f, 0.0f, 0.0f);   // По центру
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI / 2.0f,
                angle / (float)Math.PI * 4.0f) *
                Matrix.Translation(4.0f, 0.0f, 0.0f);   // Сдвиг вправо
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 4.0f,
                angle / (float)Math.PI / 2.0f) *
                Matrix.Translation(-4.0f, 0.0f, 0.0f);   // Сдвиг влево
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            //
            // Нижний ряд
            //
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 2.0f,
                angle / (float)Math.PI / 4.0f) *
                Matrix.Translation(0.0f, -4.0f, 0.0f);   // По центру
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI / 2.0f,
                angle / (float)Math.PI * 4.0f) *
                Matrix.Translation(4.0f, -4.0f, 0.0f);   // Сдвиг вправо
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 4.0f,
                angle / (float)Math.PI / 2.0f) *
                Matrix.Translation(-4.0f, -4.0f, 0.0f);   // Сдвиг влево
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            //
            // Верхний ряд
            //
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 2.0f,
                angle / (float)Math.PI / 4.0f) *
                Matrix.Translation(0.0f, 4.0f, 0.0f);   // По центру
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI / 2.0f,
                angle / (float)Math.PI * 4.0f) *
                Matrix.Translation(4.0f, 4.0f, 0.0f);   // Сдвиг вправо
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 4.0f,
                angle / (float)Math.PI / 2.0f) *
                Matrix.Translation(-4.0f, 4.0f, 0.0f);   // Сдвиг влево
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            // Дополнительные кубы
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 2.0f,
                angle / (float)Math.PI / 4.0f) *
                Matrix.Translation(
                    0.0f,
                    (float)Math.Cos(angle),
                    (float)Math.Sin(angle));
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI / 2.0f,
                angle / (float)Math.PI * 4.0f) *
                Matrix.Translation(
                    4.0f,
                    (float)Math.Sin(angle),
                    (float)Math.Cos(angle));
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            device.Transform.World = Matrix.RotationYawPitchRoll(
                angle / (float)Math.PI,
                angle / (float)Math.PI * 2.0f,
                angle / (float)Math.PI / 2.0f) *
                Matrix.Translation(
                -4.0f,
                (float)Math.Cos(angle),
                (float)Math.Sin(angle));
            device.DrawIndexedPrimitives(
                PrimitiveType.TriangleList, 0, 0, 8, 0, indices.Length / 3);
    
            if (flagRotate)
                angle += 0.02F;
        }
    }
}